@@ -50,8 +50,6 @@ DEFAULT_USER_AGENT = 'keystoneauth1/%s %s %s/%s' % (
 # here and we'll add it to the list as required.
 _LOG_CONTENT_TYPES = set(['application/json'])
 
-_logger = utils.get_logger(__name__)
-
 
 def _construct_session(session_obj=None):
     # NOTE(morganfainberg): if the logic in this function changes be sure to
@@ -159,7 +157,7 @@ def _determine_calling_package():
 
 
 def _determine_user_agent():
-    """Attempt to programatically generate a user agent string.
+    """Attempt to programmatically generate a user agent string.
 
     First, look at the name of the process. Return this unless it is in
     the `ignored` list.  Otherwise, look at the function call stack and
@@ -244,6 +242,8 @@ class Session(object):
                                  that do not share an auth plugin, it can
                                  be provided here. (optional, defaults to
                                  None which means automatically manage)
+    :param bool split_loggers: Split the logging of requests across multiple
+                               loggers instead of just one. Defaults to False.
     """
 
     user_agent = None
@@ -256,7 +256,7 @@ class Session(object):
                  cert=None, timeout=None, user_agent=None,
                  redirect=_DEFAULT_REDIRECT_LIMIT, additional_headers=None,
                  app_name=None, app_version=None, additional_user_agent=None,
-                 discovery_cache=None):
+                 discovery_cache=None, split_loggers=None):
 
         self.auth = auth
         self.session = _construct_session(session)
@@ -273,6 +273,9 @@ class Session(object):
         if discovery_cache is None:
             discovery_cache = {}
         self._discovery_cache = discovery_cache
+        # NOTE(mordred) split_loggers kwarg default is None rather than False
+        # so we can distinguish between the value being set or not.
+        self._split_loggers = split_loggers
 
         if timeout is not None:
             self.timeout = float(timeout)
@@ -324,16 +327,44 @@ class Session(object):
             return (header[0], '{SHA1}%s' % token_hash)
         return header
 
+    def _get_split_loggers(self, split_loggers):
+        """Get a boolean value from the various argument sources.
+
+        We default split_loggers to None in the kwargs of the Session
+        constructor so we can track set vs. not set. We also accept
+        split_loggers as a parameter in a few other places. In each place
+        we want the parameter, if given by the user, to win.
+        """
+        # None is the default value in each method's kwarg. None means "unset".
+        if split_loggers is None:
+            # If no value was given, try the value set on the instance.
+            split_loggers = self._split_loggers
+        if split_loggers is None:
+            # If neither a value was given on the method, nor a value was given
+            # on the Session constructor, then the value is False.
+            split_loggers = False
+        return split_loggers
+
     def _http_log_request(self, url, method=None, data=None,
                           json=None, headers=None, query_params=None,
-                          logger=_logger):
+                          logger=None, split_loggers=None):
+        string_parts = []
+
+        if self._get_split_loggers(split_loggers):
+            logger = utils.get_logger(__name__ + '.request')
+        else:
+            # Only a single logger was passed in, use string prefixing.
+            string_parts.append('REQ:')
+            if not logger:
+                logger = utils.get_logger(__name__)
+
         if not logger.isEnabledFor(logging.DEBUG):
             # NOTE(morganfainberg): This whole debug section is expensive,
             # there is no need to do the work if we're not going to emit a
             # debug log.
             return
 
-        string_parts = ['REQ: curl -g -i']
+        string_parts.append('curl -g -i')
 
         # NOTE(jamielennox): None means let requests do its default validation
         # so we need to actually check that this is False.
@@ -356,7 +387,8 @@ class Session(object):
             string_parts.append(url)
 
         if headers:
-            for header in headers.items():
+            # Sort headers so that testing can work consistently.
+            for header in sorted(headers.items()):
                 string_parts.append('-H "%s: %s"'
                                     % self._process_header(header))
         if json:
@@ -373,7 +405,18 @@ class Session(object):
 
     def _http_log_response(self, response=None, json=None,
                            status_code=None, headers=None, text=None,
-                           logger=_logger):
+                           logger=None, split_loggers=True):
+        string_parts = []
+        body_parts = []
+        if self._get_split_loggers(split_loggers):
+            logger = utils.get_logger(__name__ + '.response')
+            body_logger = utils.get_logger(__name__ + '.body')
+        else:
+            # Only a single logger was passed in, use string prefixing.
+            string_parts.append('RESP:')
+            body_parts.append('RESP BODY:')
+            body_logger = logger
+
         if not logger.isEnabledFor(logging.DEBUG):
             return
 
@@ -382,6 +425,19 @@ class Session(object):
                 status_code = response.status_code
             if not headers:
                 headers = response.headers
+
+        if status_code:
+            string_parts.append('[%s]' % status_code)
+        if headers:
+            # Sort headers so that testing can work consistently.
+            for header in sorted(headers.items()):
+                string_parts.append('%s: %s' % self._process_header(header))
+        logger.debug(' '.join(string_parts))
+
+        if not body_logger.isEnabledFor(logging.DEBUG):
+            return
+
+        if response is not None:
             if not text:
                 # NOTE(samueldmq): If the response does not provide enough info
                 # about the content type to decide whether it is useful and
@@ -406,17 +462,9 @@ class Session(object):
         if json:
             text = self._json.encode(json)
 
-        string_parts = ['RESP:']
-
-        if status_code:
-            string_parts.append('[%s]' % status_code)
-        if headers:
-            for header in headers.items():
-                string_parts.append('%s: %s' % self._process_header(header))
         if text:
-            string_parts.append('\nRESP BODY: %s\n' % text)
-
-        logger.debug(' '.join(string_parts))
+            body_parts.append(text)
+            body_logger.debug(' '.join(body_parts))
 
     @staticmethod
     def _set_microversion_headers(
@@ -450,8 +498,8 @@ class Session(object):
         # TODO(mordred) cinder uses volume in its microversion header. This
         # logic should be handled in the future by os-service-types but for
         # now hard-code for cinder.
-        if (service_type.startswith('volume')
-                or service_type == 'block-storage'):
+        if (service_type.startswith('volume') or
+                service_type == 'block-storage'):
             service_type = 'volume'
         headers.setdefault('OpenStack-API-Version',
                            '{service_type} {microversion}'.format(
@@ -465,7 +513,7 @@ class Session(object):
                 user_agent=None, redirect=None, authenticated=None,
                 endpoint_filter=None, auth=None, requests_auth=None,
                 raise_exc=True, allow_reauth=True, log=True,
-                endpoint_override=None, connect_retries=0, logger=_logger,
+                endpoint_override=None, connect_retries=0, logger=None,
                 allow=None, client_name=None, client_version=None,
                 microversion=None, microversion_service_type=None,
                 **kwargs):
@@ -560,6 +608,14 @@ class Session(object):
 
         :returns: The response to the request.
         """
+        # If a logger is passed in, use it and do not log requests, responses
+        # and bodies separately.
+        if logger:
+            split_loggers = False
+        else:
+            split_loggers = None
+        logger = logger or utils.get_logger(__name__)
+
         headers = kwargs.setdefault('headers', dict())
         if microversion:
             self._set_microversion_headers(
@@ -623,14 +679,14 @@ class Session(object):
             elif self.app_name:
                 agent.append(self.app_name)
 
-            for additional in self.additional_user_agent:
-                agent.append('%s/%s' % additional)
-
             if client_name and client_version:
                 agent.append('%s/%s' % (client_name, client_version))
             elif client_name:
                 agent.append(client_name)
 
+            for additional in self.additional_user_agent:
+                agent.append('%s/%s' % additional)
+
             if not agent:
                 # NOTE(jamielennox): determine_user_agent will return an empty
                 # string on failure so checking for None will ensure it is only
@@ -671,7 +727,7 @@ class Session(object):
                                    data=kwargs.get('data'),
                                    headers=headers,
                                    query_params=query_params,
-                                   logger=logger)
+                                   logger=logger, split_loggers=split_loggers)
 
         # Force disable requests redirect handling. We will manage this below.
         kwargs['allow_redirects'] = False
@@ -681,7 +737,7 @@ class Session(object):
 
         send = functools.partial(self._send_request,
                                  url, method, redirect, log, logger,
-                                 connect_retries)
+                                 split_loggers, connect_retries)
 
         try:
             connection_params = self.get_auth_connection_params(auth=auth)
@@ -713,13 +769,29 @@ class Session(object):
             request_id = (resp.headers.get('x-openstack-request-id') or
                           resp.headers.get('x-compute-request-id'))
             if request_id:
-                logger.debug('%(method)s call to %(service_name)s for '
-                             '%(url)s used request id '
-                             '%(response_request_id)s',
-                             {'method': resp.request.method,
-                              'service_name': service_name,
-                              'url': resp.url,
-                              'response_request_id': request_id})
+                if self._get_split_loggers(split_loggers):
+                    id_logger = utils.get_logger(__name__ + '.request-id')
+                else:
+                    id_logger = logger
+                if service_name:
+                    id_logger.debug(
+                        '%(method)s call to %(service_name)s for '
+                        '%(url)s used request id '
+                        '%(response_request_id)s', {
+                            'method': resp.request.method,
+                            'service_name': service_name,
+                            'url': resp.url,
+                            'response_request_id': request_id
+                        })
+                else:
+                    id_logger.debug(
+                        '%(method)s call to '
+                        '%(url)s used request id '
+                        '%(response_request_id)s', {
+                            'method': resp.request.method,
+                            'url': resp.url,
+                            'response_request_id': request_id
+                        })
 
         # handle getting a 401 Unauthorized response by invalidating the plugin
         # and then retrying the request. This is only tried once.
@@ -738,7 +810,7 @@ class Session(object):
 
         return resp
 
-    def _send_request(self, url, method, redirect, log, logger,
+    def _send_request(self, url, method, redirect, log, logger, split_loggers,
                       connect_retries, connect_retry_delay=0.5, **kwargs):
         # NOTE(jamielennox): We handle redirection manually because the
         # requests lib follows some browser patterns where it will redirect
@@ -784,13 +856,15 @@ class Session(object):
             time.sleep(connect_retry_delay)
 
             return self._send_request(
-                url, method, redirect, log, logger,
+                url, method, redirect, log, logger, split_loggers,
                 connect_retries=connect_retries - 1,
                 connect_retry_delay=connect_retry_delay * 2,
                 **kwargs)
 
         if log:
-            self._http_log_response(response=resp, logger=logger)
+            self._http_log_response(
+                response=resp, logger=logger,
+                split_loggers=split_loggers)
 
         if resp.status_code in self._REDIRECT_STATUSES:
             # be careful here in python True == 1 and False == 0
@@ -812,7 +886,7 @@ class Session(object):
                 # NOTE(jamielennox): We don't pass through connect_retry_delay.
                 # This request actually worked so we can reset the delay count.
                 new_resp = self._send_request(
-                    location, method, redirect, log, logger,
+                    location, method, redirect, log, logger, split_loggers,
                     connect_retries=connect_retries,
                     **kwargs)
 
@@ -974,6 +1048,30 @@ class Session(object):
         auth = self._auth_required(auth, 'determine endpoint URL')
         return auth.get_api_major_version(self, **kwargs)
 
+    def get_all_version_data(self, auth=None, interface=None,
+                             region_name=None, **kwargs):
+        """Get version data for all services in the catalog.
+
+        :param auth:
+            The auth plugin to use for token. Overrides the plugin on
+            the session. (optional)
+        :type auth: keystoneauth1.plugin.BaseAuthPlugin
+        :param interface:
+            Type of endpoint to get version data for. Can be a single value
+            or a list of values. A value of None indicates that all interfaces
+            should be queried. (optional, defaults to public)
+        :param string region_name:
+            Region of endpoints to get version data for. A valueof None
+            indicates that all regions should be queried. (optional, defaults
+            to None)
+        :returns: A dictionary keyed by region_name with values containing
+            dictionaries keyed by interface with values being a list of
+            `~keystoneauth1.discover.VersionData`.
+        """
+        auth = self._auth_required(auth, 'determine endpoint URL')
+        return auth.get_all_version_data(
+            self, interface=interface, region_name=region_name, **kwargs)
+
     def get_auth_connection_params(self, auth=None, **kwargs):
         """Return auth connection params as provided by the auth plugin.
 
@@ -1074,7 +1172,6 @@ class Session(object):
         auth = self._auth_required(auth, 'get project_id')
         return auth.get_project_id(self)
 
-
 REQUESTS_VERSION = tuple(int(v) for v in requests.__version__.split('.'))
 
 
